// MZSplit - a small and simple file splitter
//
// Copyright 2005-2007 Marcello Zaniboni
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, version 2.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Pixmap.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Int_Input.H>
#include <FL/Fl_Round_Button.H>
#include <FL/Fl_Check_Button.H>
#include <FL/Fl_Return_Button.H>
#include <FL/Fl_Progress.H>
#include <FL/fl_ask.H>
#include <FL/x.H>

#include <cstring>

#include "resource.h"
#include "logo.h"
#include "callbacks.h"


int make_windows(const char *, int, char **);

int main(int argc, char **argv) {
    Fl::scheme("plastic");
    fl_message_font(FL_HELVETICA, 12);
    return make_windows(argc == 2 ? (const char *) argv[1] : "", 1, argv);
}

int make_windows(const char *default_input_file_name, int argc, char **argv) {

    /* MAIN WINDOW */

    Fl_Window main_window(415 + 3, 235 + 2, "MZSplit 3.0");

    // logo
    Fl_Pixmap logo_image_pixmap(logo_image);
    Fl_Box logo(280, 180, 130, 50);
    logo.image(logo_image_pixmap);

    // file name
    Fl_Box lbl_filename(5, 5 + 2, 190, 20, "File name:");
    lbl_filename.labelsize(12);
    lbl_filename.align(FL_ALIGN_TOP_LEFT|FL_ALIGN_INSIDE);
    Fl_Input input_filename(5, 25, 270, 20);
    input_filename.labelsize(12);
    input_filename.textsize(12);
    if (strlen(default_input_file_name) > 0) input_filename.value(default_input_file_name);
    Fl_Button btn_browse_file(280, 15, 130, 30, "Browse...");
    btn_browse_file.labelsize(12);
    btn_browse_file.callback((Fl_Callback *)cb_file_browsing, &input_filename);

    // output directory
    Fl_Box lbl_foldername(5, 55 + 2, 190, 20, "Output folder:");
    lbl_foldername.labelsize(12);
    lbl_foldername.align(FL_ALIGN_TOP_LEFT|FL_ALIGN_INSIDE);
    Fl_Check_Button chk_defaultfolder(125, 55, 150, 20, " use input folder");
    chk_defaultfolder.labelsize(12);
    chk_defaultfolder.value(1);
    chk_defaultfolder.callback((Fl_Callback *)cb_default_folder_behaviour, &chk_defaultfolder);
    Fl_Input input_foldername(5, 75, 270, 20);
    input_foldername.labelsize(12);
    input_foldername.textsize(12);
    input_foldername.deactivate();
    Fl_Button btn_browse_folder(280, 65, 130, 30, "Browse...");
    btn_browse_folder.labelsize(12);
    btn_browse_folder.deactivate();
    btn_browse_folder.callback((Fl_Callback *)cb_folder_browsing, &input_foldername);

    // file size
    Fl_Box lbl_filesize(5, 105 + 2, 150, 20, "File size:");
    lbl_filesize.labelsize(12);
    lbl_filesize.align(FL_ALIGN_TOP_LEFT|FL_ALIGN_INSIDE);
    Fl_Box box_filesize(5, 125, 270, 70);
    box_filesize.box(FL_ENGRAVED_FRAME);
    box_filesize.labelsize(12);
    box_filesize.align(FL_ALIGN_TOP_LEFT);
    Fl_Round_Button round_splitsize1(10, 130, 260, 20, "Floppy disk (1.44 MB)");
    round_splitsize1.type(FL_RADIO_BUTTON);
    round_splitsize1.down_box(FL_ROUND_DOWN_BOX);
    round_splitsize1.labelsize(12);
    round_splitsize1.callback((Fl_Callback *)cb_radio_behaviour);
    Fl_Round_Button round_splitsize2(10, 150, 260, 20, "CD-ROM (650 MB)");
    round_splitsize2.type(FL_RADIO_BUTTON);
    round_splitsize2.down_box(FL_ROUND_DOWN_BOX);
    round_splitsize2.labelsize(12);
    round_splitsize2.callback((Fl_Callback *)cb_radio_behaviour);
    Fl_Round_Button round_splitsize3(10, 170, 200, 20, "Custom size (MiB)");
    round_splitsize3.type(FL_RADIO_BUTTON);
    round_splitsize3.down_box(FL_ROUND_DOWN_BOX);
    round_splitsize3.labelsize(12);
    round_splitsize3.value(1);
    round_splitsize3.callback((Fl_Callback *)cb_radio_behaviour);
    Fl_Int_Input input_customsize(160, 170, 110, 20);
    input_customsize.labelsize(12);
    input_customsize.textsize(12);
    input_customsize.static_value("256");
    input_customsize.callback((Fl_Callback *)cb_custom_size_behaviour);

    // buttons
    Fl_Return_Button btn_go(5, 200, 270, 30, "Go");
    btn_go.labelsize(12);
    btn_go.callback((Fl_Callback *)cb_go);
    Fl_Button btn_help(280, 110, 130, 30, "Help");
    btn_help.labelsize(12);
    btn_help.callback((Fl_Callback *)cb_help);
    Fl_Button btn_about(280, 145, 130, 30, "License");
    btn_about.labelsize(12);
    btn_about.callback((Fl_Callback *)cb_about);

    main_window.end();
    main_window.icon((char *)LoadIcon(fl_display, MAKEINTRESOURCE(IDI_ICON1)));
    main_window.callback((Fl_Callback *)cb_no_esc_key_window);
    main_window.show(1, argv);

    /* SPLIT & JOIN WINDOW */

    Fl_Window progress_window(415 + 3, 105 + 2, "MZSplit 3.0 - work in progress; please wait...");

    Fl_Box lbl_currentfilename(5, 5 + 2, 405, 20, "Processing file");
    lbl_currentfilename.labelsize(12);
    lbl_currentfilename.align(FL_ALIGN_TOP_LEFT|FL_ALIGN_INSIDE);

    Fl_Input output_currentfilename(5, 25, 405, 20);
    output_currentfilename.textsize(12);
    output_currentfilename.align(FL_ALIGN_TOP_LEFT|FL_ALIGN_INSIDE);
    output_currentfilename.value("?");
    output_currentfilename.position(strlen(output_currentfilename.value()), strlen(output_currentfilename.value()));
    output_currentfilename.readonly(1);

    Fl_Progress pb_progress(5, 60, 405, 40, "0 %");
    pb_progress.labelsize(14);
    pb_progress.value(0);

    progress_window.end();
    progress_window.icon((char *)LoadIcon(fl_display, MAKEINTRESOURCE(IDI_ICON1)));
    progress_window.cursor(FL_CURSOR_WAIT); // ################ does it work?

    init_references(
        &main_window,
        &round_splitsize1,
        &round_splitsize2,
        &round_splitsize3,
        &input_customsize,
        &input_filename,
        &input_foldername,
        &btn_browse_folder,
        &progress_window,
        &output_currentfilename,
        &pb_progress);

    return(Fl::run());
}
